@@ -1,7 +1,9 @@
-/* New simplified depmod without backwards compat stuff and not
-   requiring ksyms.
-
-   (C) 2002 Rusty Russell IBM Corporation
+/*
+ * New simplified depmod without backwards compat stuff and not
+ * requiring ksyms.
+ *
+ * (C) 2010 Jon Masters <jcm@jonmasters.org>, and others.
+ * (C) 2002 Rusty Russell IBM Corporation
  */
 #define _GNU_SOURCE /* asprintf */
 
@@ -876,13 +878,13 @@ static int output_aliases(struct module *modules, FILE *out, char *dirname)
 		filename2modname(modname, i->pathname);
 
 		/* Grab from old-style .modalias section. */
-		tbl = file->ops->load_strings(file, ".modalias", NULL, fatal);
+		tbl = file->ops->load_strings(file, ".modalias", NULL);
 		for (j = 0; tbl && j < tbl->cnt; j++)
 			fprintf(out, "alias %s %s\n", tbl->str[j], modname);
 		strtbl_free(tbl);
 
 		/* Grab from new-style .modinfo section. */
-		tbl = file->ops->load_strings(file, ".modinfo", NULL, fatal);
+		tbl = file->ops->load_strings(file, ".modinfo", NULL);
 		for (j = 0; tbl && j < tbl->cnt; j++) {
 			const char *p = tbl->str[j];
 			if (strstarts(p, "alias="))
@@ -913,7 +915,7 @@ static int output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 		filename2modname(modname, i->pathname);
 
 		/* Grab from old-style .modalias section. */
-		tbl = file->ops->load_strings(file, ".modalias", NULL, fatal);
+		tbl = file->ops->load_strings(file, ".modalias", NULL);
 		for (j = 0; tbl && j < tbl->cnt; j++) {
 			alias = NOFAIL(strdup(tbl->str[j]));
 			underscores(alias);
@@ -926,7 +928,7 @@ static int output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 		strtbl_free(tbl);
 
 		/* Grab from new-style .modinfo section. */
-		tbl = file->ops->load_strings(file, ".modinfo", NULL, fatal);
+		tbl = file->ops->load_strings(file, ".modinfo", NULL);
 		for (j = 0; tbl && j < tbl->cnt; j++) {
 			const char *p = tbl->str[j];
 			if (strstarts(p, "alias=")) {
@@ -948,6 +950,72 @@ static int output_aliases_bin(struct module *modules, FILE *out, char *dirname)
 	return 1;
 }
 
+static int output_softdeps(struct module *modules, FILE *out, char *dirname)
+{
+	struct module *i;
+	struct elf_file *file;
+	struct string_table *tbl;
+	int j;
+
+	fprintf(out, "# Soft dependencies extracted from modules themselves.\n");
+	fprintf(out, "# Copy, with a .conf extension, to /etc/modprobe.d to use "
+		"it with modprobe.\n");
+	for (i = modules; i; i = i->next) {
+		char modname[strlen(i->pathname)+1];
+
+		file = i->file;
+		filename2modname(modname, i->pathname);
+
+		/* Grab from new-style .modinfo section. */
+		tbl = file->ops->load_strings(file, ".modinfo", NULL);
+		for (j = 0; tbl && j < tbl->cnt; j++) {
+			const char *p = tbl->str[j];
+			if (strstarts(p, "softdep="))
+				fprintf(out, "softdep %s %s\n",
+					modname, p + strlen("softdep="));
+		}
+		strtbl_free(tbl);
+	}
+	return 1;
+}
+
+static int output_devname(struct module *modules, FILE *out, char *dirname)
+{
+	struct module *m;
+
+	fprintf(out, "# Device nodes to trigger on-demand module loading.\n");
+	for (m = modules; m != NULL; m = m->next) {
+		struct string_table *tbl;
+		int i;
+		char type = '\0';
+		const char *devname = NULL;
+
+		tbl = m->file->ops->load_strings(m->file, ".modinfo", NULL);
+		for (i = 0; tbl && i < tbl->cnt; i++) {
+			const char *p = tbl->str[i];
+			unsigned int maj, min;
+
+			if (sscanf(p, "alias=char-major-%u-%u", &maj, &min) == 2)
+				type = 'c';
+			else if (sscanf(p, "alias=block-major-%u-%u", &maj, &min) == 2)
+				type = 'b';
+			else if (strstarts(p, "alias=devname:"))
+				devname = &p[strlen("alias=devname:")];
+
+			if (type && devname) {
+				char modname[strlen(m->pathname)+1];
+
+				filename2modname(modname, m->pathname);
+				fprintf(out, "%s %s %c%u:%u\n",
+					modname, devname, type, maj, min);
+				break;
+			}
+		}
+		strtbl_free(tbl);
+	}
+	return 1;
+}
+
 struct depfile {
 	char *name;
 	int (*func)(struct module *, FILE *, char *dirname);
@@ -967,9 +1035,11 @@ static struct depfile depfiles[] = {
 	{ "modules.seriomap", output_serio_table, 1 },
 	{ "modules.alias", output_aliases, 0 },
 	{ "modules.alias.bin", output_aliases_bin, 0 },
+	{ "modules.softdep", output_softdeps, 0 },
 	{ "modules.symbols", output_symbols, 0 },
 	{ "modules.symbols.bin", output_symbols_bin, 0 },
 	{ "modules.builtin.bin", output_builtin_bin, 0 },
+	{ "modules.devname", output_devname, 0 },
 };
 
 /* If we can't figure it out, it's safe to say "true". */
@@ -1120,8 +1190,7 @@ static int parse_config_file(const char *filename,
 			version = strsep_skipspace(&ptr, "\t ");
 			subdir = strsep_skipspace(&ptr, "\t ");
 
-			if (strcmp(version, kernelversion) != 0 &&
-			    strcmp(version, "*") != 0)
+			if (!regex_match(kernelversion, (const char *)version))
 				continue;
 
 			nofail_asprintf(&pathname, "%s%s%s/%s/%s.ko", basedir,
@@ -1342,7 +1411,7 @@ int main(int argc, char *argv[])
 	else if (system_map)
 		load_system_map(system_map);
 	else if (print_unknown) {
-		warn("-e needs -E or -F");
+		warn("-e needs -E or -F\n");
 		print_unknown = 0;
 	}
 
