@@ -242,7 +242,7 @@ static struct elf_file *grab_module(const char *name,
 			module = grab_elf_file(filename);
 			if (!module)
 				error("modinfo: could not open %s: %s\n",
-					*filename, strerror(errno));
+					 filename, strerror(errno));
 			free(filename);
 			return module;
 		}
@@ -307,7 +307,7 @@ int main(int argc, char *argv[])
 			ret = 1;
 			continue;
 		}
-		tags = mod->ops->load_strings(mod, ".modinfo", NULL, error);
+		tags = mod->ops->load_strings(mod, ".modinfo", NULL);
 		if (!tags) {
 			release_elf_file(mod);
 			continue;
